{ Copyright (C) 1997-2004 John Coppens (john@jcoppens.com)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Library General Public License for more details.
  
  You should have received a copy of the GNU Library General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
} 

unit campic;

interface

uses
  i2c;

const
  PIC_ADDR	= $0B;

  PIC_GETID	= $00;
  PIC_READPROG	= $01;
  PIC_WRITEPROG = $02;
  PIC_READEE	= $04;
  PIC_WRITEEE	= $05;

type
  i2c_pic = object (i2c_port)
    procedure Initialize(port_addr: word);
    procedure Release;
    
    function  ChecksumOk(chks: byte): boolean;
    function  BlockSent(str: string): boolean;
    function  BlockReceived(var str: string): boolean;
    function  HaveID(var minor, major: word): boolean;
    function  ProgAreaRead(beg, len: longint; var bff): boolean;
    function  DataAreaRead(beg, len: longint; var bff): boolean;
  end;

implementation

procedure i2c_pic.Initialize(port_addr: word);
begin
  inherited Initialize(port_addr, PIC_ADDR);
end;

procedure i2c_pic.Release;
begin
  inherited Release;
end;

function i2c_pic.ChecksumOk(chks: byte): boolean;
begin
  ChecksumOk := (chks and $ff) = 0;
end;

{ BlockSent:
    Return true if the string has been sent correctly
    The string must contain the command, and optionally data bytes
    BlockSent adds the length count before and the checksum after
    the packet.
}

function i2c_pic.BlockSent(str: string): boolean;
var
  c: integer;
  chks: byte;
begin
  BlockSent := false;
  chks := length(str) + 1;
  if ByteSent(length(str) + 1) then begin	{ Send the packet length }
    for c := 1 to length(str) do begin
      if not ByteSent(ord(str[c])) then
        exit;
      chks := chks + ord(str[c]);
    end;
    BlockSent := ByteSent((256 - chks) and $ff);
  end;
end;

{ BlockReceived:
    Receive a packet from the peripheral.
    Return true if a valid packet has been received (in str)
    Receives the packet length, then so many bytes as necessary
    for that count.
    Check the checksum before approving the packet.
    The output string does NOT contain the pkt_len byte or checksum.
}

function i2c_pic.BlockReceived(var str: string): boolean;
var
  len, b, nr, chks: byte;
begin
  BlockReceived := false;

  str := '';
  if ByteReceived(len, true) then begin		{ always receive length }
    chks := len;
    for nr := 1 to len do begin
      if not ByteReceived(b, nr <> len) then	{ confirm all but last byte }
        exit;
      str := str + chr(b);
      chks := chks + b;
    end;
    if chks = 0 then begin
      writeln('--- chks: ', chks);
      BlockReceived := true;
      dec(str[0]);
    end;
  end;
end;

{ HaveID:
    Interrogate the Version of the bootloader software in the peripheral
}

function i2c_pic.HaveID(var minor, major: word): boolean;
var
  blk: string;
begin
  Start;
  HaveID := false;
  if WriteStarted then
    if BlockSent(chr(PIC_GETID)) then begin
      Stop;
      Start;
      if ReadStarted then
        if BlockReceived(blk) then begin
          minor := ord(blk[2]);
          major := ord(blk[3]);
          HaveID := true;
        end;
    end;
  Stop;
end;


function  i2c_pic.ProgAreaRead(beg, len: longint; var bff): boolean;
var
  str: string;
  c: integer;
  arr: array [0..31] of word absolute bff;
begin
  Start;
  ProgAreaRead := false;
  if WriteStarted then
    if BlockSent(chr(PIC_READPROG) + chr(len)			// data length
                                   + chr(beg and $ff)		// offset LSB
                                   + chr((beg shr 8) and $ff)	// offset MSB
                                   + chr(0)) then begin		// offset USB
      Stop;
      Start;
      if ReadStarted then
        if BlockReceived(str) then begin
          for c := 0 to len-1 do
            arr[c] := ord(str[2*c + 6]) + 256*ord(str[2*c + 7]);
          ProgAreaRead := true;
        end;
    end;
  Stop;
end;
    

function  i2c_pic.DataAreaRead(beg, len: longint; var bff): boolean;
var
  str: string;
  c: integer;
  arr: array [0..31] of byte absolute bff;
begin
  Start;
  DataAreaRead := false;
  if WriteStarted then
    if BlockSent(chr(PIC_READEE) + chr(len)			// data length
                                 + chr(beg and $ff)		// offset LSB
                                 + chr((beg shr 8) and $ff)	// offset MSB
                                 + chr(0)) then begin		// offset USB
      Stop;
      Start;
      if ReadStarted then
        if BlockReceived(str) then begin
          for c := 0 to len-1 do
            arr[c] := ord(str[c + 6]);
          DataAreaRead := true;
        end;
    end;
  Stop;
end;
    

begin
end.
