{ Copyright (C) 1997-2004 John Coppens (john@jcoppens.com)
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Library General Public License for more details.
  
  You should have received a copy of the GNU Library General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
} 

{ Program to control the camera controller }

program CamControl;

uses
  campic, crt;

const
  PARPORT = $378;
  PROGBFFLEN = 32;
  DATABFFLEN = 32;
  
var
  cam: i2c_pic;

procedure GetID;
var
  minor, major: word;
begin
  if cam.HaveID(minor, major) then
    writeln('Bootloader firmware: ', major, '.', minor);
end;

procedure ReadProgArea;
var
  bff: array [0..PROGBFFLEN-1]  of word;
  c: integer;
  beg, len, ofs, blklen, remaining: longint;
begin
  writeln('Start offset - '); readln(beg);
  writeln('Length - '); readln(len);
  writeln;
  writeln('Program memory:');
  writeln;
  ofs := beg;
  remaining := len;
  while ofs < beg+len do begin
    blklen := PROGBFFLEN;
    if remaining < PROGBFFLEN then
       blklen := remaining;
    remaining := remaining - blklen;
    if cam.ProgAreaRead(ofs, blklen, bff) then begin
      for c := 0 to blklen-1 do begin
        if (c mod 8) = 0 then begin
          writeln;
          write(hexstr(ofs + c, 4), '  ');
        end;
        write(hexstr(bff[c], 4), ' ');
      end;
    end;
    ofs := ofs + blklen;
  end;
  writeln;
end;

procedure ReadDataArea;
var
  bff: array [0..DATABFFLEN-1]  of word;
  c: integer;
  beg, len, ofs, blklen, remaining: longint;
begin
  write('Start offset - '); readln(beg);
  write('Length - '); readln(len);
  writeln;
  writeln('Data memory:');
  writeln;
  ofs := beg;
  remaining := len;
  while ofs < beg+len do begin
    blklen := DATABFFLEN;
    if remaining < DATABFFLEN then
       blklen := remaining;
    remaining := remaining - blklen;
    if cam.DataAreaRead(ofs, blklen, bff) then begin
      for c := 0 to blklen-1 do begin
        if (c mod 8) = 0 then begin
          writeln;
          write(hexstr(ofs + c, 4), '  ');
        end;
        write(hexstr(bff[c], 2), ' ');
      end;
    end;
    ofs := ofs + blklen;
  end;
  writeln;
end;

procedure DLProgArea;
begin
end;

procedure DLDataArea;
begin
end;


procedure ULProgArea;        
begin
end;

procedure ULDataArea;        
begin
end;


procedure StartUserProgram;
begin
end;


procedure DecodeParameters;
begin
end;

var
  cmd, scmd: char;

begin
  cam.Initialize(PARPORT);
  
  if cam.Responds then
    writeln('Camera board responds.');
  
  if paramcount = 0 then begin
    writeln('Interactive mode');
    repeat
      writeln;
      writeln('[I|i] Get firmware version');
      writeln('[R]   Read memory');
      writeln('[D]   Download memory to file');
      writeln('[U]   Upload from file');
      writeln('[G]   Start user program');
      writeln('[X]   Exit program');
      writeln;
      write('Command - ');
      cmd := upcase(readkey); writeln(cmd);
      case cmd of
        'I': GetID;
        'R': begin
               write('Read [P]rogram, [D]ata area - ');
               scmd := upcase(readkey); writeln(scmd);
               case scmd of
                 'P': ReadProgArea;
                 'D': ReadDataArea;
               end;
             end;
        'D': begin
               write('Download [P]rogram, [D]ata area - ');
               scmd := upcase(readkey); writeln(scmd);
               case scmd of
                 'P': DLProgArea;
                 'D': DLDataArea;
               end;
             end;
        'U': begin
               write('Upload [P]rogram, [D]ata area - ');
               scmd := upcase(readkey); writeln(scmd);
               case scmd of
                 'P': ULProgArea;
                 'D': ULDataArea;
               end;
             end;
        'G': StartUserProgram;
      end;
    until cmd = 'X';
  end else
    DecodeParameters;
  
  cam.Release;
end.
